/***************************************************************************
                          sglwindow.cpp  -  description
                             -------------------
    begin                : Wed Jan 23 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
    url                  : www.steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sglwindow.h"
#include <SDL.h>
#include <string.h>
#include <stdlib.h> //for (s)rand()
#include <time.h> //for time()

/** DEFINES/GLOBALS **/
const float CUBE_SIZE = 1.0f;
const float CUBE_ROT_SPEEDX = 0.1f;
const float CUBE_ROT_SPEEDY = 0.1f;
const float ORBIT_ROT_SPEED = 0.1f;

//lights on
const float WHOLE_SCENE_LIGHT[] = { 0.5f, 0.5f, 0.5f, 1.0f};
const float DIFFUSE_LIGHT[]  = { 0.6f, 0.6f, 0.6f, 1.0f};
const float SPECULAR_LIGHT[] = {-.1f, -0.1f, -0.1f, 1.0f};
const float LIGHT_POSITION1[] = { 0.0f, 0.0f,15.0f, 1.0f};
const float SPOT_DIRECTION[] = {0.0f,0.0f,-1.0f};
const float MATERIAL[] = {1.0f,1.0f,1.0f,1.0f};

/** INLINE FUNCTIONS **/

void fillColourArray(sColor* colors, sColor color)
{
	for (int i = 0; i < TEX_ARRAY_SIZE; ++i)
	{
		colors[i] = color;
	}
}

/******************************************************************************/

SGLWindow::SGLWindow()
{
	cubeRotation[0] = 0.0f;
	cubeRotation[1] = 0.0f;
	sphere = 0;
	textures[0] = 0;
	textures[1] = 0;
	orbitRotation = 0.0f;

	//fill texture data arrays with black
	sColor color = { 0, 0, 0 };
	fillColourArray(texArrayOne, color);
	fillColourArray(texArrayTwo, color);

	//init random number generator
	srand(time(NULL));
}
SGLWindow::~SGLWindow()
{
	//cleanup textures and GLU quadric
	//objects
	if (textures[0] != 0)
	{
		glDeleteTextures(2,textures);
	}

	if (sphere != 0)
	{
		gluDeleteQuadric(sphere);
	}
}
/** No descriptions */
void SGLWindow::init()
{
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glEnable(GL_TEXTURE_2D);

	/** texture setup **/
	glGenTextures(2,textures);

	//ONE
	glBindTexture(GL_TEXTURE_2D,textures[0]);
	//set parameters
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	//initial texture
	glTexImage2D(GL_TEXTURE_2D,0,3,TEX_X,
			TEX_Y,0,GL_RGB,
			GL_UNSIGNED_BYTE,(char*)texArrayOne);

	//TWO
	//do the same as before..
	glBindTexture(GL_TEXTURE_2D,textures[1]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D,0,3,TEX_X,
			TEX_Y,0,GL_RGB,
			GL_UNSIGNED_BYTE,(char*)texArrayTwo);

	/**
	* LIGHT
	*********/

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,WHOLE_SCENE_LIGHT);

	//Main
	glLightfv(GL_LIGHT0,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT0,GL_POSITION,LIGHT_POSITION1);

	//Spotlight
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f); //60 Spotlight
	glLightf(GL_LIGHT0,GL_SPOT_EXPONENT, 60.0f);
	glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,SPOT_DIRECTION);

	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

	//Materials
	glMaterialfv(GL_FRONT,GL_SPECULAR,MATERIAL);
	glMaterialf(GL_FRONT,GL_SHININESS,32.0f);

	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	/***
	 * setup sphere
	 ******/
	sphere = gluNewQuadric();
	//set normals to smooth and create texture
	//coords too
	gluQuadricNormals(sphere, GLU_SMOOTH);
	gluQuadricTexture(sphere, GLU_TRUE);

}
/** No descriptions */
void SGLWindow::resize(int width, int height)
{
	glViewport(0,0,width,height);

	if (height == 0)
		height = 1;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(54.0f,width/height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}
/** No descriptions */
void SGLWindow::update(float frametime)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

	/** update texture before drawing object **/
	updateTexture();

	//setup camera
	glTranslatef(0.0f,0.0f,-7.0f);

	/**
	 update positions and rotations
	 **/
	cubeRotation[0] += CUBE_ROT_SPEEDX * frametime;
	cubeRotation[1] += CUBE_ROT_SPEEDY * frametime;
	orbitRotation += ORBIT_ROT_SPEED * frametime;

	/*
	  two objects are drawn: a sphere with texture 2 and
	  a cube with texture 1. These objects will be rotated on an orbit
	  of 5.0f radius. Both objects have an angle of 180 to each other.
	 */

	//go to position of first object
	glRotatef(orbitRotation,0.0f,1.0f,0.0f);
	glTranslatef(0.0f,0.0f,2.0f);
	//draw cube here
	drawCube();

	//go to position of second object, 180 from first, so
	//it's a straight line
	glTranslatef(0.0f,0.0f,-4.0f);
	drawSphere();
}

void SGLWindow::keyDown(int key)
{
	switch(key)
 	{
    	case SDLK_ESCAPE:
     		SDL_Event sent;
       		sent.type = SDL_QUIT;

         	SDL_PushEvent(&sent);
          	break;
        }
}

/*!
    \fn SGLWindow::updateTexture()
 */
void SGLWindow::updateTexture()
{
	/** update textures
	 **/

	/** texture one **/

	//some static informations for texture 1
	static int position1 = 0;
	static sColor color1 = { 0,0,127 };

	//set color of current pixel
	texArrayOne[position1] = color1;
	//next texture pixel
	position1 += 3;

	//when end of texture has been reached, reset to 0
	//and create new random color
	if (position1 >= TEX_ARRAY_SIZE)
	{
		position1 = 0;
		color1.red = rand()% 255;
		color1.green = rand() %255;
		color1.blue = rand() % 255;
	}


	/** texture two **/

	//the second texture is simply a random
	//field of randomly coloured pixels

	//get random pixel
	int randomPixel = rand() % TEX_ARRAY_SIZE;

	texArrayTwo[randomPixel].red = rand() % 255;
	texArrayTwo[randomPixel].green = rand() % 255;
	texArrayTwo[randomPixel].blue = rand() % 255;

	/** re-create/update textures
	     animation effect
	 **/

	//ONE
	glBindTexture(GL_TEXTURE_2D, textures[0]);

	glTexSubImage2D(GL_TEXTURE_2D,
			  0,
			  0,
			  0,
			  TEX_X,
			  TEX_Y,
			  GL_RGB,
			  GL_UNSIGNED_BYTE,
			  (char*)texArrayOne);
	//TWO
	glBindTexture(GL_TEXTURE_2D, textures[1]);

	glTexSubImage2D(GL_TEXTURE_2D,
			  0,
			  0,
			  0,
			  TEX_X,
			  TEX_Y,
			  GL_RGB,
			  GL_UNSIGNED_BYTE,
			  (char*)texArrayTwo);
}

/*!
    \fn SGLWindow::drawCube()
 */
void SGLWindow::drawCube()
{
	glBindTexture(GL_TEXTURE_2D, textures[0]);

	glPushMatrix();

	glRotatef(cubeRotation[0],1.0f,0.0f,0.0f);
	glRotatef(cubeRotation[1],0.0f,1.0f,0.0f);

	glBegin(GL_QUADS);
		glColor3f(1.0f,1.0f,1.0f);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 4 (Front)

		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 4 (Back)

		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 4 (Top)

		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 3 (Bottom)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 4 (Bottom)

		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 4 (Right)

		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-CUBE_SIZE, -CUBE_SIZE,  CUBE_SIZE);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE,  CUBE_SIZE);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-CUBE_SIZE,  CUBE_SIZE, -CUBE_SIZE);	// Point 4 (Left)
	glEnd();

	glPopMatrix();
}

/*!
    \fn SGLWindow::drawSphere()
 */
void SGLWindow::drawSphere()
{
	glBindTexture(GL_TEXTURE_2D, textures[1]);

	gluSphere(sphere,1.0f,30,30);
}




